@@ -624,7 +624,12 @@ static void on_surface_configure_finished(ctx_t * ctx) {
 #else
     xdg_surface_ack_configure(ctx->wl.xdg_surface, ctx->wl.last_surface_serial);
 #endif
-    wl_surface_commit(ctx->wl.surface);
+
+    // draw frame to attach and commit buffer
+    // reduces number of empty commits
+    // required if libdecor is used
+    // contains a surface commit, no second commit necessary
+    wlm_egl_draw_frame(ctx);
 
     // reset configure sequence state machine
 #ifndef WITH_LIBDECOR
@@ -707,9 +712,7 @@ static void on_libdecor_frame_configure(
         wp_viewport_set_destination(ctx->wl.viewport, width, height);
 
         // resize window to reflect new surface size
-        if (ctx->egl.initialized) {
-            wlm_egl_resize_window(ctx);
-        }
+        wlm_egl_resize_window(ctx);
     }
 
     // update configure sequence state machine
@@ -803,9 +806,7 @@ static void on_xdg_toplevel_configure(
         wp_viewport_set_destination(ctx->wl.viewport, width, height);
 
         // resize window to reflect new surface size
-        if (ctx->egl.initialized) {
-            wlm_egl_resize_window(ctx);
-        }
+        wlm_egl_resize_window(ctx);
     }
 
     // update configure sequence state machine
@@ -1036,6 +1037,15 @@ void wlm_wayland_init(ctx_t * ctx) {
 
         wp_fractional_scale_v1_add_listener(ctx->wl.fractional_scale, &fractional_scale_listener, (void *)ctx);
     }
+}
+
+// --- configure_window ---
+
+void wlm_wayland_configure_window(struct ctx * ctx) {
+    if (!ctx->egl.initialized) {
+        wlm_log_error("wayland::configure_window(): egl must be initialized first\n");
+        wlm_exit_fail(ctx);
+    }
 
 #if WITH_LIBDECOR
     // create libdecor context
@@ -1051,14 +1061,12 @@ void wlm_wayland_init(ctx_t * ctx) {
 
     // set libdecor app properties
     libdecor_frame_set_app_id(ctx->wl.libdecor_frame, "at.yrlf.wl_mirror");
-    libdecor_frame_set_title(ctx->wl.libdecor_frame, "Wayland Output Mirror");
+    wlm_mirror_update_title(ctx);
 
     // map libdecor frame
+    // commits surface and triggers configure sequence
     libdecor_frame_map(ctx->wl.libdecor_frame);
 
-    // commit surface to trigger configure sequence
-    wl_surface_commit(ctx->wl.surface);
-
     // wait for events
     // - expecting libdecor frame configure event
     wl_display_roundtrip(ctx->wl.display);
@@ -1088,7 +1096,7 @@ void wlm_wayland_init(ctx_t * ctx) {
 
     // set xdg toplevel properties
     xdg_toplevel_set_app_id(ctx->wl.xdg_toplevel, "at.yrlf.wl_mirror");
-    xdg_toplevel_set_title(ctx->wl.xdg_toplevel, "Wayland Output Mirror");
+    wlm_mirror_update_title(ctx);
 
     // commit surface to trigger configure sequence
     wl_surface_commit(ctx->wl.surface);
@@ -1099,16 +1107,31 @@ void wlm_wayland_init(ctx_t * ctx) {
     wl_display_roundtrip(ctx->wl.display);
 #endif
 
+    // wait until surface is configured
+    // - if not, attempt to wait another roundtrip
+    // - this can happen with e.g. libdecor, when it doesn't find any plugins
+    if (!ctx->wl.configured) {
+        wlm_log_warn("wayland::configure_window(): surface not configured, attempting to wait until configure event\n");
+
+        int round_trip_count = 1;
+        while (!ctx->wl.configured) {
+            round_trip_count++;
+            wl_display_roundtrip(ctx->wl.display);
+        }
+
+        wlm_log_debug(ctx, "wayland::configure_window(): surface configured after %d round trips\n", round_trip_count);
+    }
+
     // check if surface is configured
     // - expecting surface to be configured at this point
     if (!ctx->wl.configured) {
-        wlm_log_error("wayland::init(): surface not configured\n");
+        wlm_log_error("wayland::configure_window(): surface not configured, exiting\n");
         wlm_exit_fail(ctx);
     }
 
     // set fullscreen on target output if requested by initial options
     if (ctx->opt.fullscreen && ctx->opt.fullscreen_output != NULL) {
-        wlm_log_debug(ctx, "wayland::init(): fullscreening on target output\n");
+        wlm_log_debug(ctx, "wayland::configure_window(): fullscreening on target output\n");
         wlm_wayland_window_set_fullscreen(ctx);
     }
 }
